﻿using UnityEngine;
using UnityEngine.UI;

public enum TruncationMethod
{
	None,
	CharacterClip,
	WordBoundary,
}

public static class UITextExtensions
{
	const char CHAR_SPACE = ' ';
	const string STRING_SPACE = " ";

	public static string TruncateWithSuffix(this Text textComponent, TruncationMethod truncationMethod = TruncationMethod.CharacterClip, string suffix = "...")
	{
		string result = textComponent.text;

		if (!(textComponent.horizontalOverflow == HorizontalWrapMode.Overflow ||
		          textComponent.verticalOverflow == VerticalWrapMode.Overflow ||
		          string.IsNullOrEmpty(textComponent.text)))
		{
			Vector2 size = textComponent.rectTransform.rect.size;
			TextGenerationSettings settings = textComponent.GetGenerationSettings(size);
			TextGenerator generator = textComponent.cachedTextGenerator;
			generator.Populate(textComponent.text, settings);
            
			int numCharsVisible = generator.characterCountVisible;
			int totalChars = Mathf.Max(generator.characterCount - 1, 0); // totalChars is always numCharsVisible + 1; this fixes that.
			if (numCharsVisible < totalChars)
			{
				// Get the visible string
				string visibleText = textComponent.text.Substring(0, Mathf.Max(numCharsVisible - 1, 0));
                
				switch (truncationMethod)
				{
					case TruncationMethod.WordBoundary:
                        // Look for a previous word boundary to place the suffix
                        // If the suffix is longer than the # of chars after the last index of ' ',
                        // we need to keep dialing it back to previous words and check again
						while (visibleText.Length > 0)
						{
							int charsAvailableForSuffix = visibleText.Length - visibleText.LastIndexOf(CHAR_SPACE);
							if (charsAvailableForSuffix >= suffix.Length)
							{
								visibleText = visibleText.Substring(0, visibleText.LastIndexOf(CHAR_SPACE));
								result = string.Concat(visibleText, suffix);
								break;
							}

							if (!visibleText.Contains(STRING_SPACE))
							{
								result = visibleText;
								break;
							}   

							visibleText = visibleText.Substring(0, visibleText.LastIndexOf(CHAR_SPACE) - 1);
						}
						break;
                        
					default:
						return TrimCharacters(visibleText, suffix);
				}
			}
		}
        
		return result;
	}

	static string TrimCharacters(string visibleText, string suffix)
	{
		// generate a substring
		int deltaLength = Mathf.Max(visibleText.Length - suffix.Length, 0);
		string substring = visibleText.Substring(0, deltaLength);
        
		// while the last character is not a letter, remove it
		char lastChar = GetLastChar(substring);
		while (!char.IsLetter(lastChar) && lastChar != default(char))
		{
			substring = substring.Remove(substring.Length - 1, 1);
			lastChar = GetLastChar(substring);
		}
        
		// trim the end of the substring before appending the ellipsis.
		substring.TrimEnd();
		substring = string.Concat(substring, suffix);
		return substring;
	}

	static char GetLastChar(string value)
	{
		return value.Length > 0 ? value [value.Length - 1] : default(char);
	}
}